from sqlalchemy import Column, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import declarative_mixin


@declarative_mixin
class UUIDMixin:
    """
    Mixin to provide a uuid primary key, based on
    https://www.postgresql.org/docs/current/functions-uuid.html

    - example usage::

        from accentdatabase.base import Base
        from accentdatabase.mixins import UUIDMixin

        class MyTable(UUIDMixin, Base)
            pass

    """

    id = Column(
        UUID(as_uuid=True),
        primary_key=True,
        server_default=text("gen_random_uuid()"),
    )



from typing import Union

from pydantic import BaseSettings, PostgresDsn, PyObject


class AppConfig(BaseSettings):
    url: Union[PostgresDsn, str]
    future: bool = True
    json_serializer: PyObject = "accentdatabase.encoders.json_serializer"
    echo: bool = False

    class Config:
        env_prefix = "database_"


config = AppConfig()



import json

import pydantic.json


def json_serializer(*args, **kwargs) -> str:
    """
    Encodes json in the same way that pydantic does.
    https://pydantic-docs.helpmanual.io/usage/dataclasses/#json-dumping
    """

    return json.dumps(
        *args,
        default=pydantic.json.pydantic_encoder,
        **kwargs,
    )
